; TASM cross-assembler definitions
;
#define  db    .byte
#define  dw    .word
#define  ds    .block
#define  org   .org
#define  end   .end
#define DEFB   .BYTE
#define DEFW   .WORD
#define EQU    .EQU



; A useful reference
;
;  ____0___1___2___3___4___5___6___7___8___9___A___B___C___D___E___F____
;  00 SPC GRA GRA GRA GRA GRA GRA GRA GRA GRA GRA  "  GBP  $   :   ?  0F
;  10  (   )   >   <   =   +   -   *   /   ;   ,   .   0   1   2   3  1F
;  20  4   5   6   7   8   9   A   B   C   D   E   F   G   H   I   J  2F
;  30  K   L   M   N   O   P   Q   R   S   T   U   V   W   X   Y   Z  3F


   ; -------------=-=-=-=-=-=-=-=-=-=-=-------------
   ;               ASSEMBLER  TEMPLATE
   ; -------------=-=-=-=-=-=-=-=-=-=-=-------------

PRT0 EQU $00
PRT1 EQU $20
PRT2 EQU $40
PRT3 EQU $60
PRT4 EQU $80
PRT5 EQU $a0
PRT6 EQU $c0
PRT7 EQU $e0

#define PRT2BC(port)      ld bc,port*256+7           ; interface port address | IO port address
#define PREP_BFR_READ     PRT2BC(PRT0)\ ld a,$00\ out (c),a\ ld b,PRT1
#define PREP_BFR_WRITE    PRT2BC(PRT0)\ ld a,$ff\ out (c),a\ ld b,PRT2
#define BFR_READ    PRT2BC(PRT1)\ in a,(c)
#define BFR_WRITE      PRT2BC(PRT2)\ out (c),a        ; value in A
#define DIR_OPEN    PRT2BC(PRT3)\ ld a,$00\ out (c),a
#define DIR_GET_NEXT   PRT2BC(PRT3)\ ld a,$ff\ out (c),a
#define FILE_OPEN_READ    PRT2BC(PRT4)\ ld a,$00\ out (c),a
#define FILE_OPEN_WRITE   PRT2BC(PRT4)\ ld a,$01\ out (c),a
#define FILE_OPEN_READ80   PRT2BC(PRT4)\ ld a,$02\ out (c),a
#define FILE_OPEN_WRITE80  PRT2BC(PRT4)\ ld a,$03\ out (c),a
#define FILE_GET_SIZE     PRT2BC(PRT4)\ ld a,$10\ out (c),a
#define FILE_CLOSE     PRT2BC(PRT4)\ ld a,$80\ out (c),a
#define FILE_DELETE    PRT2BC(PRT4)\ ld a,$f0\ out (c),a
#define FILE_READ   PRT2BC(PRT5)\ out (c),a        ; bytes to read into globalbuffer
#define FILE_WRITE     PRT2BC(PRT6)\ out (c),a        ; bytes to write to file from globalbuffer
#define CMD_PARSE   PRT2BC(PRT7)\ ld a,$00\ out (c),a
#define DISABLE_EXT_ROM   PRT2BC(PRT7)\ ld a,$b0\ out (c),a
#define DISABLE_EXT_ROM_DELAY   PRT2BC(PRT7)\ ld a,$f0\ out (c),a



         org   $4009



ERR_NR   .equ  $4000
FLAGS    .equ  $4001
ERR_SP   .equ  $4002
RAMTOP   .equ  $4004
MODE     .equ  $4006
PPC      .equ  $4007

;= System variables as saved ============================================

VERSN:   db 0
E_PPC:   dw 0
D_FILE:  dw dfile
DF_CC:   dw dfile+1
VARS:    dw var
DEST:    dw 0
E_LINE:  dw var+1
CH_ADD:  dw last-1
X_PTR:   dw 0
STKBOT:  dw last
STKEND:  dw last
BERG:    db 0
MEM:     dw MEMBOT
         db 0
DF_SZ:   db 2
S_TOP:   dw 1
LAST_K:  db $FF,$FF,$FF
MARGIN:  db 55
NXTLIN:  dw line1
OLDPPC:  dw 0
FLAGX:   db 0
STRLEN:  dw 0
T_ADDR:  dw $0C8D
SEED:    dw 0
FRAMES:  dw $FFFF
COORDS:  db 0,0
PR_CC:   db $BC
S_POSN:  db 33,24
CDFLAG:  db 01000000B
PRTBUF:  ds 33       ; print buffer - iow scratch ram for us :)
MEMBOT:  ds 30       ; calculator's scratch
         ds 2

line0:
   db $00,$00     ; line number
   dw line1-$-2   ; line length

   db $ea         ; REM



   jr    findfirst
   jr    findnext



   
findfirst:
   ; determine what variable to get initial directory from.
   ; if variable not found then assume initial directory is current.

   ; code is modified from the z88 zx81 library
   
   ld    e,$26          ; 'A'
   call  findvar        ; C clear if found, HL points to name

   ld    de,0           ; source dir length
   jr    c,ff_sendstr

   inc   hl
   ld    e,(hl)         ; recover length
   inc   hl             ; point to data
   inc   hl

ff_sendstr:
   call  sendstr        ; hl -> name, de = len
   DIR_OPEN
   call  response40


findnext:
   DIR_GET_NEXT

   call  response

   ld    bc,0           ; indicate nothing found
   cp    $3f            ; done?
   ret   z

   call  expect_40

   ; there's an entry - read it into the printer buffer,
   ; then stuff it into the supplied var.
   
   PREP_BFR_READ
   call     interIOdelay

   ld    hl,PRTBUF      ; destination

   PRT2BC(PRT1)

ff_readitem:
   in    a,(c)
   ld    (hl),a

   and   a              ; zero terminated - tostr function expects zt string
   jr    z,ff_copyit

   inc   hl
   jr    ff_readitem

ff_copyit:
   ld    e,$26          ; copy it to target string
   ld    hl,PRTBUF
   call  setstr
   ret



;;
;; a small delay in between interface writes
;;
interIOdelay:
   ld    a,8

iw_delay:
   dec   a
   jr    nz,iw_delay
   ret




sendstr:
   PREP_BFR_WRITE

sds_loop:
   ld    a,e            ; assume a len < 256 ;)
   and   a
   jr    nz,sds_onemore

   out   (c),a
   ret

sds_onemore:
   ld    a,(hl)
   out   (c),a
   inc   hl
   dec   e
   jr    sds_loop




;; wait for a slow command to finish
;;  bit 7 = busy
;;  bit 6 = error
;;
response:
   ld    a,0            ; delay loop setting - so we don't steal all PIC's cycles by polling...

rsp_delay:
   dec   a
   jr    nz,rsp_delay

   in    a,(c)          ; get response code
   jp    m,rsp_delay    ; and loop whilst negative

   ret



response40:
   call  response


;;
;; test for a 40 return code, show error if bits 0..5 incl are not 0
;;
expect_40:
   cp    $40            ; usually all ok if $3f,
   ret   z

   ; else fall through to error generation code
   ; first discard old return address the put main-exec there

   pop   hl             ; discard old returnadress
   add   a,$3f          ; inverse 0..7
   ld    hl,-5          ; store fake instruction just below stack
   adc   hl,sp          ; leaving room for RST return address
   ld    (hl),$cf
   inc   hl
   ld    (hl),a
   dec   hl
   jp    (hl)


   


;
;  ZX 81 specific routines
;  by Stefano Bodrato, Oct 2007
;
;  Copy a string to a BASIC variable

; enter : hl = char *value
;          e = char variable

setstr:
   ld    (ss_pointer+1),hl

   call  findvar
   jr    c,ss_store     ; variable not found

   call  $09F2          ; get next variable start
   call  $0A60          ; reclaim space (delete)

ss_store:
   ld    bc,0

ss_pointer:
   ld    de,0           ; point to the string
   push  de

ss_lenloop:
   inc   bc             ; string length counter
   inc   de
   ld    a,(de)
   and   a
   jr    nz,ss_lenloop

   push  hl
   push  bc 
   inc   bc
   inc   bc
   inc   bc
   call  $099E             ; MAKE-ROOM
   pop   bc
   pop   hl

   ld    a,(fv_morevar+1)
   ld    (hl),a
   inc   hl
   ld    (hl),c
   inc   hl
   ld    (hl),b
   inc   hl
   pop   de

   ex    de,hl

   ldir
   dec   bc                ; indicate a string was created
   ret





findvar:
   ld    a,e
   add   a,$20

   ld    (fv_morevar+1),a

   ld    hl,($4010)     ; VARS

fv_loop:
   ld    a,(hl)
   cp    128
   jr    nz,fv_morevar

   scf                  ; not found - return with C flag set
   ret

fv_morevar:
   cp    0              ; !! SELF MODIFIES !!
   ret   z              ; found - return with C clear

   call  $09F2          ;get next variable start
   ex    de,hl
   jr    fv_loop

;
;
   db $76               ; end of line for REM
;
;


line1:

;- Display file --------------------------------------------

dfile:
   db $76
   db $76,$76,$76,$76,$76,$76,$76,$76
   db $76,$76,$76,$76,$76,$76,$76,$76
   db $76,$76,$76,$76,$76,$76,$76,$76

;- BASIC-Variables ----------------------------------------

var:
   db $80

;- End of program area ----------------------------

last:

   end
