 
;;; PASMO assembler format


; top 3 bits are latch address, 7 is IO port address
;
PRT0 EQU $0007
PRT1 EQU $2007
PRT2 EQU $4007
PRT3 EQU $6007
PRT4 EQU $8007
PRT5 EQU $a007
PRT6 EQU $c007
PRT7 EQU $e007



;; THE LATEST ZXPAND ROM EXPORTS THESE ENTRY POINTS.
;;  This library exports the functionality from the ROM,
;;  and will behave identically.
;;
;; api_responder  .equ  $1ff6
;; api_fileop     .equ  $1ff8
;; api_sendstring .equ  $1ffa
;; api_xfer       .equ  $1ffc
;; api_rdjoy      .equ  $1ffe




api_rdjoy:
   ld    bc,PRT7
   ld    a,$a0
   out   (c),a
   call  interwritedelay
   in    a,(c)

   ld    c,a
   ld    b,0
   ret



api_xfer:
   and   a
   jp    z,readdatablock

   ; falls into...

writeblk:
   call  prepbfrwrite

wb_loop:
   ld    a,(de)
   out   (c),a
   inc   de
   dec   l
   jr    nz,wb_loop
   ret





;; push high-bit-terminated string at (de) to the interface as zero-terminated string
;; if de = 0, then only terminating zero is pushed
;;
api_sendstring:
   call  prepbfrwrite
   ld    a,e
   or    d
   jr    z,ss_terminate

ss_nameoutloop:
   ld    a,(de)         ; pic firmware will strip bit 7
   out   (c),a
   inc   de
   and   a
   jp    p,ss_nameoutloop

ss_terminate:
   xor   a              ; write terminating 0
   out   (c),a

   ret



api_fileop:
   and   a
   jp    z,loadfilentry
   jp    p,deletefileentry
   jp    savefilentry




api_responder:
   call  response
   ld    b,0
   ld    c,a
   ret









;; wait for a slow command to finish
;; bit 7 = busy
;; bit 6 = error
;;
response:
   ld    a,0            ; delay loop setting

rsp_delay:
   dec   a
   jr    nz,rsp_delay

   in    a,(c)          ; get response code
   jp    m,rsp_delay    ; and loop whilst negative

   ret





initfileoprd:
   xor   a              ; read

initfileop:
   push  af
   call  api_sendstring ; push the file path at (de) - sets up C for us
   pop   af

   ld    bc,PRT4        ; a = 0 for read, 1 for write
   out   (c),a

   call  response40
   jp    retrieve_meta





disable_ovl:
   ld    bc,PRT7
   ld    a,$b0
   out   (c),a
   ret
 


loadfilentry:
   ; de points to load filename string

   call  initfileoprd   ; send filename, open file, retrieve meta

   ld    a,(16444+4)    ; flags returned by interface - nonzero will disable ROM
   or    a
   jr    z,LOAD_2

   ld    hl,disable_ovl ; return via 'disable overlay ROM'
   push  hl             ; fake return address on the stack

LOAD_2:
   ld    hl,(16444)     ; data length
   push  hl 

   ld    hl,(16444+2)   ; load address

   jr    ld_remaining

   ;

ld_readdata256:
   xor   a

ld_readdata:
   push  de

   ld    e,a            ; loop count

   ld    bc,PRT5
   out   (c),a          ; bytes to read into globalbuffer 0 = 256

   call  response40

   ; data is ready for us

   call  prepbfrread

ld_getbyte:
   in    a,(c)
   ld    (hl),a
   inc   hl
   dec   e
   jr    nz,ld_getbyte

ld_remaining:
   pop   de
   dec   d
   jp    p,ld_readdata256

ld_cleanup:
   ld    a,e            ; any stragglers?
   ld    e,0
   or    a
   jr    nz,ld_readdata


close40:
   ld    bc,PRT4        ; close file
   ld    a,$80
   out   (c),a

   ; exit via response 40


response40:
   call  response

;; test for a 40 return code, show error if bits 0..5 incl are not 0
;;
expect_40:
   cp    $40            ; all ok
   ret   z

   ; else fall through to error generation code

   add   a,$3f ; inverse 0..7
   ld    hl,-5 ; store fake instruction just below stack
   adc   hl,sp ; leaving room for RST return address
   ld    (hl),$cf
   inc   hl
   ld    (hl),a
   dec   hl
   jp    (hl)










savefilentry:
   ld    a,1
   call  initfileop

sv_calclen:
   ld    hl,(16444)     ; data length
   ld    a,h
   or    l
   jr    nz,sv_lenspecified

   ld    hl,($4014)
   ld    de,(16444+2)

   and   a
   sbc   hl,de

sv_lenspecified:
   push  hl

   ld    hl,(16444+2) ; save address
   jr    sv_remaining

   ;

sv_writedata256:
   xor   a

sv_writedata:
   push  de

   push  af
   ld    e,a            ; loop count

   call  prepbfrwrite

sv_getbyte:
   ld    a,(hl)
   out   (c),a
   inc   hl
   dec   e
   jr    nz,sv_getbyte

   pop   af

   ld    bc,PRT6
   out   (c),a          ; bytes to write to file from globalbuffer

   call  response40


sv_remaining:
   pop   de
   dec   d
   jp    p,sv_writedata256

sv_cleanup:
   ld    a,e            ; any stragglers?
   ld    e,0
   or    a
   jr    nz,sv_writedata

   jr    close40





retrieve_meta:
   ; retrieve file metadata
   ; SIZE (W) + LOADADDR (W) + FLAGS (W) + 26 more

   ld    de,16444 ; PRT-BUFF
   ld    l,32

   ; read data to memory at (de) of length l

readdatablock:
   call  prepbfrread

rtv_meta:
   in    a,(c)
   ld    (de),a
   inc   de
   dec   l
   jr    nz,rtv_meta

   ret




deletefileentry:
   call  api_sendstring ; push the file path at (de) - sets up C for us

   ld    bc,PRT4
   ld    a,$f0
   out   (c),a
   jp    response





prepbfrwrite:
   ld    bc,PRT0
   ld    a,$ff
   out   (c),a
   ld    b,(PRT2/256)
   jr    interwritedelay


prepbfrread:
   ld    bc,PRT0
   ld    a,$00
   out   (c),a
   ld    b,(PRT1/256)

   ; falls though into...

;; a small delay in between interface writes
;;
interwritedelay:
   ld    a,8

iw_delay:
   dec   a
   jr    nz,iw_delay
   ret
